<?php

/**
 * @file
 * View hook implementations for Similar entries module.
 */

/**
 * Implements hook_views_data().
 */
function similar_views_data() {
  // We namespace the data keys to prevent conflicts with 'similar'.
  $data['similar_entries']['table']['group'] = t('Similar entries');
  $data['similar_entries']['table']['join'] = array('#global' => array());

  // Adds the similarity argument handler.
  $data['similar_entries']['similarity'] = array(
    'title' => t('Similarity score'),
    'help' => t('The rating given to the related content given as an argument.'),
    'filter' => array(
      'handler' => 'similar_handler_filter_similarity',
      'help' => t('Filter by similarity score. Note: this is not accurate when the <i>include content fields</i> option is enabled in the <i>Similar entries: Nid</i> argument.'),
      'label' => 'Score',
    ),
    'field' => array(
      'handler' => 'similar_handler_field_similarity',
      'click sortable' => TRUE,
    ),
    'sort' => array('handler' => 'similar_handler_sort_similarity'),
  );
  $data['similar_entries']['nid'] = array(
    'title' => t('Nid'),
    'help' => t('ID of content item(s). Passes content IDs to Similar entries.'),
    'argument' => array(
      'handler' => 'similar_handler_argument_nid',
      'name field' => 'title',
      'numeric' => TRUE,
      'validate type' => 'nid',
    ),
  );

  // Create relationships for each indexed table.
  foreach (similar_get_indices() as $table => $fields) {
    $data[$table]['table']['group'] = t('Similar entries');
    $data[$table]['table']['join']['node'] = array(
      'table' => $table,
      'left_field' => 'vid',
      'field' => 'vid',
    );
    $data[$table]['table']['join']['node_revisions'] = array(
      'table' => $table,
      'left_field' => 'vid',
      'field' => 'vid',
    );
  }

  return $data;
}

/**
 * Implements hook_views_handlers().
 */
function similar_views_handlers() {
  return array(
    'info' => array('path' => drupal_get_path('module', 'similar') . '/views'),
    'handlers' => array(
      'similar_handler_sort_similarity' => array(
        'parent' => 'views_handler_sort',
      ),
      'similar_handler_argument_nid' => array(
        'parent' => 'views_handler_argument_numeric',
      ),
      'similar_handler_filter_similarity' => array(
        'parent' => 'views_handler_filter_numeric',
      ),
      'similar_handler_field_similarity' => array(
        'parent' => 'views_handler_field',
      ),
    ),
  );
}
